// https://flowbite.com/docs/components/timeline/
// TODO : Timeline
// activity timeline (custom)

use maud::{html, PreEscaped, Render};

use crate::ui::UIWidget;

pub struct TimelineElement {
    icon: Option<Box<dyn UIWidget>>,
    time_heading: bool,
    title: Box<dyn UIWidget>,
    time: String,
    body: Box<dyn UIWidget>,
}

impl TimelineElement {
    pub fn new<T: UIWidget + 'static, B: UIWidget + 'static>(time: &str, title: T, body: B) -> Self {
        TimelineElement { icon: None, time_heading: false, title: Box::new(title), time: time.to_string(), body: Box::new(body) }
    }

    pub fn time_heading(mut self) -> Self {
        self.time_heading = true;
        self
    }

    pub fn icon<T: UIWidget + 'static>(mut self, icon: T) -> Self {
        self.icon = Some(Box::new(icon));
        self
    }
}

impl Render for TimelineElement {
    fn render(&self) -> maud::Markup {
        html! {
            li class="mb-10 ms-6" {
                @if let Some(icon) = &self.icon {
                span class="absolute flex items-center justify-center w-6 h-6 bg-blue-100 rounded-full -start-3 dark:ring-gray-900 dark:bg-blue-900" {
                    (icon)
                };
            } @else {
                div class="absolute w-3 h-3 bg-gray-200 rounded-full mt-1.5 -start-1.5 border border-white dark:border-gray-900 dark:bg-gray-700" {};
            }

            @if self.time_heading {
                time class="mb-1 text-sm font-normal leading-none text-gray-400 dark:text-gray-500" { (self.time) };
                h3 class="text-lg font-semibold text-gray-900 dark:text-white" { (self.title) };
            } @else {
                h3 class="mb-1 text-lg font-semibold text-gray-900 dark:text-white" { (self.title) };
                time class="block mb-2 text-sm font-normal leading-none text-gray-400 dark:text-gray-500" { (self.time) };
            }

                (self.body)
                };
        }
    }
}

pub struct TimelineWidget {
    vanish: bool,
    elements: Vec<TimelineElement>,
}

impl TimelineWidget {
    pub fn add_element(mut self, element: TimelineElement) -> Self {
        self.elements.push(element);
        self
    }

    pub fn vanish(mut self) -> Self {
        self.vanish = true;
        self
    }
}

impl Render for TimelineWidget {
    fn render(&self) -> maud::Markup {
        self.render_with_class("")
    }
}

impl UIWidget for TimelineWidget {
    fn can_inherit(&self) -> bool {
        false
    }

    fn base_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn extended_class(&self) -> Vec<String> {
        Vec::new()
    }

    fn render_with_class(&self, _: &str) -> maud::Markup {
        if self.vanish {
            return html! {
                @for element in &self.elements {
                    (element);
                }
            };
        }

        html! {
            ol class="ml-6 mt-4 relative border-s border-gray-200 dark:border-gray-700" {
                @for element in &self.elements {
                    (element);
                }
            };
        }
    }
}

#[allow(non_snake_case)]
pub fn Timeline() -> TimelineWidget {
    TimelineWidget {
        vanish: false,
        elements: Vec::new(),
    }
}

// TODO : https://flowbite.com/docs/components/timeline/#activity-log
/*
pub fn ActivityLog(items: Vec<TimelineElement>) -> PreEscaped<String> {
    html! {
        ol class="relative border-s border-gray-200 dark:border-gray-700" {
<li class="mb-10 ms-6">            
    <span class="absolute flex items-center justify-center w-6 h-6 bg-blue-100 rounded-full -start-3 ring-8 ring-white dark:ring-gray-900 dark:bg-blue-900">
        <img class="rounded-full shadow-lg" src="/docs/images/people/profile-picture-3.jpg" alt="Bonnie image"/>
    </span>
    <div class="items-center justify-between p-4 bg-white border border-gray-200 rounded-lg shadow-xs sm:flex dark:bg-gray-700 dark:border-gray-600">
        <time class="mb-1 text-xs font-normal text-gray-400 sm:order-last sm:mb-0">just now</time>
        <div class="text-sm font-normal text-gray-500 dark:text-gray-300">Bonnie moved <a href="#" class="font-semibold text-blue-600 dark:text-blue-500 hover:underline">Jese Leos</a> to <span class="bg-gray-100 text-gray-800 text-xs font-normal me-2 px-2.5 py-0.5 rounded-sm dark:bg-gray-600 dark:text-gray-300">Funny Group</span></div>
    </div>
</li>
<li class="mb-10 ms-6">
    <span class="absolute flex items-center justify-center w-6 h-6 bg-blue-100 rounded-full -start-3 ring-8 ring-white dark:ring-gray-900 dark:bg-blue-900">
        <img class="rounded-full shadow-lg" src="/docs/images/people/profile-picture-5.jpg" alt="Thomas Lean image"/>
    </span>
    <div class="p-4 bg-white border border-gray-200 rounded-lg shadow-xs dark:bg-gray-700 dark:border-gray-600">
        <div class="items-center justify-between mb-3 sm:flex">
            <time class="mb-1 text-xs font-normal text-gray-400 sm:order-last sm:mb-0">2 hours ago</time>
            <div class="text-sm font-normal text-gray-500 lex dark:text-gray-300">Thomas Lean commented on  <a href="#" class="font-semibold text-gray-900 dark:text-white hover:underline">Flowbite Pro</a></div>
        </div>
        <div class="p-3 text-xs italic font-normal text-gray-500 border border-gray-200 rounded-lg bg-gray-50 dark:bg-gray-600 dark:border-gray-500 dark:text-gray-300">Hi ya'll! I wanted to share a webinar zeroheight is having regarding how to best measure your design system! This is the second session of our new webinar series on #DesignSystems discussions where we'll be speaking about Measurement.</div>
    </div>
</li>
<li class="ms-6">
    <span class="absolute flex items-center justify-center w-6 h-6 bg-blue-100 rounded-full -start-3 ring-8 ring-white dark:ring-gray-900 dark:bg-blue-900">
        <img class="rounded-full shadow-lg" src="/docs/images/people/profile-picture-1.jpg" alt="Jese Leos image"/>
    </span>
    <div class="items-center justify-between p-4 bg-white border border-gray-200 rounded-lg shadow-xs sm:flex dark:bg-gray-700 dark:border-gray-600">
        <time class="mb-1 text-xs font-normal text-gray-400 sm:order-last sm:mb-0">1 day ago</time>
        <div class="text-sm font-normal text-gray-500 lex dark:text-gray-300">Jese Leos has changed <a href="#" class="font-semibold text-blue-600 dark:text-blue-500 hover:underline">Pricing page</a> task status to  <span class="font-semibold text-gray-900 dark:text-white">Finished</span></div>
    </div>
</li>
        };
    }
}

*/
